/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "dateselect.h"
#include "globals.h"
#include "utils.h"
#include<iostream>

TDateSelect::TDateSelect():
isDateSelecting(false),
pDateSelectionDialog(0),
pDateSelectionDialogCalendar(0),
daySpinButton(0),
monthSpinButton(0),
yearSpinButton(0),
pDateSelectionDialogLabel(0),
mainButtonBox(0),
OKButton(0)
{
    Glib::RefPtr<Gnome::Glade::Xml> refXml;
    try
    {
      refXml = Gnome::Glade::Xml::create(glade_interface_path, "DateSelectionDialog");
    }
    catch(const Gnome::Glade::XmlError& ex)
    {
      std::cerr << ex.what() << std::endl;
    } 
    refXml->get_widget("DateSelectionDialog", pDateSelectionDialog);
    refXml->get_widget("SelectionCalendar", pDateSelectionDialogCalendar);
    refXml->get_widget("DaySpinButton", daySpinButton);
    refXml->get_widget("MonthSpinButton", monthSpinButton);
    refXml->get_widget("YearSpinButton", yearSpinButton);
    refXml->get_widget("DateSelectionLabel", pDateSelectionDialogLabel);

    refXml->get_widget("dialog-action_area3", mainButtonBox);
    refXml->get_widget("OKButton", OKButton);

    Gtk::Button* pButton;
    refXml->get_widget("OKButton", pButton);
    pButton->signal_clicked().connect(sigc::mem_fun(pDateSelectionDialog, &Gtk::Dialog::hide));

    refXml->get_widget("CancelButton", pButton);
    pButton->signal_clicked().connect(sigc::mem_fun(pDateSelectionDialog, &Gtk::Dialog::hide));

    pDateSelectionDialogCalendar->signal_day_selected().connect(sigc::mem_fun(*this, &TDateSelect::on_calendar_date_selected));
    pDateSelectionDialogCalendar->signal_day_selected_double_click().connect(sigc::mem_fun(*this, &TDateSelect::on_date_selection_double_click));
    daySpinButton->signal_value_changed().connect(sigc::mem_fun(*this, &TDateSelect::on_day_change));
    monthSpinButton->signal_value_changed().connect(sigc::mem_fun(*this, &TDateSelect::on_month_change));
    yearSpinButton->signal_value_changed().connect(sigc::mem_fun(*this, &TDateSelect::on_year_change));    
}


TDateSelect::~TDateSelect()
{
    if(pDateSelectionDialog) delete pDateSelectionDialog;
}

bool TDateSelect::getDateSelection(Gtk::Window* parent,
                              Glib::Date& resultDate,
                              const Glib::ustring title,
                              const Glib::ustring label,
                              const Glib::Date* defaultDate)
{
  if(pDateSelectionDialog) {
      // GTKmm provides the option to choose the calendar 
      // week starting day, however, this option, as this 
      // often happens in GTK/GTKmm, is not working.
      // Rather like the Redmond-based mantra: 
      // "You don't really need this feature..."
      // "You don't really need this feature..."
      // "You don't really need this feature..."
      // "You don't really need this feature..."
      
      pDateSelectionDialog->set_title(title);
      pDateSelectionDialogLabel->set_label(label);
      // if the default date is not specified - set the widget to today
      struct tm t_m;
      if(defaultDate) {
          defaultDate->to_struct_tm(t_m);
      } else {
          PCal::getTodayDate().to_struct_tm(t_m);
      }
      pDateSelectionDialogCalendar->select_month(t_m.tm_mon, t_m.tm_year + 1900);
      pDateSelectionDialogCalendar->select_day(t_m.tm_mday);
      pDateSelectionDialog->set_transient_for(*parent);
      
      // rearrange the buttons according to the options
      if(Gtk::Dialog::alternative_button_order(pDateSelectionDialog->get_screen())) {
          mainButtonBox->reorder_child(*OKButton, 1);
      } else {
          mainButtonBox->reorder_child(*OKButton, 2);
      }
      
      while(true) {
          int response = pDateSelectionDialog->run();
          if(response==Gtk::RESPONSE_OK) {
              guint new_year;
              guint new_month;
              guint new_day;
              pDateSelectionDialogCalendar->get_date(new_year, new_month, new_day);
              resultDate = Glib::Date(new_day, Glib::Date::Month(new_month+1), new_year);
              return true;
          } else
          if(response==Gtk::RESPONSE_HELP) {
//!!!              show_help("record pregnancy");
              continue;
          }
          break;
      }
  }
  return false;
}

void TDateSelect::on_date_selection_double_click()
{
    if(pDateSelectionDialog) {
        pDateSelectionDialog->response(Gtk::RESPONSE_OK);
        pDateSelectionDialog->hide();
    }
}

void TDateSelect::on_calendar_date_selected()
{
    if(!isDateSelecting) {
        isDateSelecting = true;
        guint new_year;
        guint new_month;
        guint new_day;
        pDateSelectionDialogCalendar->get_date(new_year, new_month, new_day);
        if(new_year<1) {
          new_year = 1;
          new_month = 0;
          pDateSelectionDialogCalendar->select_month(new_month, new_year);
        }
        daySpinButton->set_range(1, Glib::Date::get_days_in_month(Glib::Date::Month(new_month+1),new_year));
        daySpinButton->set_value(new_day);
        monthSpinButton->set_value(new_month+1);
        yearSpinButton->set_value(new_year);
        isDateSelecting = false;
    }
}

void TDateSelect::on_day_change()
{
    if(!isDateSelecting) {
        isDateSelecting = true;
        pDateSelectionDialogCalendar->select_day(daySpinButton->get_value_as_int());
        isDateSelecting = false;
    }
}
void TDateSelect::on_month_change()
{
    if(!isDateSelecting) {
        isDateSelecting = true;
        guint new_year;
        guint new_month;
        guint new_day;
        guint max_days;
        pDateSelectionDialogCalendar->get_date(new_year, new_month, new_day);
        new_month = monthSpinButton->get_value_as_int()-1;
        max_days = int(Glib::Date::get_days_in_month(Glib::Date::Month(new_month+1),new_year));
        daySpinButton->set_range(1, max_days);
        if(max_days<new_day) pDateSelectionDialogCalendar->select_day(max_days);
        pDateSelectionDialogCalendar->select_month(new_month, new_year);
        isDateSelecting = false;
    }
}
void TDateSelect::on_year_change()
{
    if(!isDateSelecting) {
        isDateSelecting = true;
        guint new_year;
        guint new_month;
        guint new_day;
        guint max_days;
        pDateSelectionDialogCalendar->get_date(new_year, new_month, new_day);
        new_year = yearSpinButton->get_value_as_int();
        max_days = int(Glib::Date::get_days_in_month(Glib::Date::Month(new_month+1),new_year));
        daySpinButton->set_range(1, max_days);
        if(max_days<new_day) pDateSelectionDialogCalendar->select_day(max_days);
        pDateSelectionDialogCalendar->select_month(new_month, new_year);
        isDateSelecting = false;
    }
}
